@@ -18,7 +18,8 @@ module Data.List.Extra(
     wordsBy, linesBy,
     breakOn, breakOnEnd, splitOn, split, chunksOf,
     -- * Basics
-    notNull, list, unsnoc, cons, snoc, drop1, mconcatMap,
+    headDef, lastDef, notNull, list, unsnoc, cons, snoc,
+    drop1, dropEnd1, mconcatMap,
     -- * Enum operations
     enumerate,
     -- * List operations
@@ -98,6 +99,26 @@ allSame [] = True
 allSame (x:xs) = all (x ==) xs
 
 
+-- | A total 'head' with a default value.
+--
+-- > headDef 1 []      == 1
+-- > headDef 1 [2,3,4] == 2
+-- > \x xs -> headDef x xs == fromMaybe x (listToMaybe xs)
+headDef :: a -> [a] -> a
+headDef d [] = d
+headDef _ (x:_) = x
+
+
+-- | A total 'last' with a default value.
+--
+-- > lastDef 1 []      == 1
+-- > lastDef 1 [2,3,4] == 4
+-- > \x xs -> lastDef x xs == last (x:xs)
+lastDef :: a -> [a] -> a
+lastDef d xs = foldl (\_ x -> x) d xs -- I know this looks weird, but apparently this is the fastest way to do this: https://hackage.haskell.org/package/base-4.12.0.0/docs/src/GHC.List.html#last
+{-# INLINE lastDef #-}
+
+
 -- | A composition of 'not' and 'null'.
 --
 -- > notNull []  == False
@@ -268,7 +289,7 @@ upper = map toUpper
 -- > \s -> fst (word1 s) == concat (take 1 $ words s)
 -- > \s -> words (snd $ word1 s) == drop 1 (words s)
 word1 :: String -> (String, String)
-word1 = second (dropWhile isSpace) . break isSpace . dropWhile isSpace
+word1 = second trimStart . break isSpace . trimStart
 
 -- | Split the first line off a string.
 --
@@ -356,20 +377,44 @@ groupOn f = groupBy ((==) `on2` f)
     where (.*.) `on2` f = \x -> let fx = f x in \y -> fx .*. f y
 
 
--- | A version of 'nub' where the equality is done on some extracted value.
+-- | /DEPRECATED/ Use 'nubOrdOn', since this function is _O(n^2)_.
+--
+--   A version of 'nub' where the equality is done on some extracted value.
 --   @nubOn f@ is equivalent to @nubBy ((==) `on` f)@, but has the
 --   performance advantage of only evaluating @f@ once for each element in the
 --   input list.
+{-# DEPRECATED nubOn "Use nubOrdOn, since this function is O(n^2)" #-}
 nubOn :: Eq b => (a -> b) -> [a] -> [a]
 nubOn f = map snd . nubBy ((==) `on` fst) . map (\x -> let y = f x in y `seq` (y, x))
 
 -- | A version of 'maximum' where the comparison is done on some extracted value.
-maximumOn :: Ord b => (a -> b) -> [a] -> a
-maximumOn f = maximumBy (compare `on` f)
+--   Raises an error if the list is empty. Only calls the function once per element.
+--
+-- > maximumOn id [] == undefined
+-- > maximumOn length ["test","extra","a"] == "extra"
+maximumOn :: (Partial, Ord b) => (a -> b) -> [a] -> a
+maximumOn f [] = error "Data.List.Extra.maximumOn: empty list"
+maximumOn f (x:xs) = g x (f x) xs
+    where
+        g v mv [] = v
+        g v mv (x:xs) | mx > mv = g x mx xs
+                      | otherwise = g v mv xs
+            where mx = f x
+
 
 -- | A version of 'minimum' where the comparison is done on some extracted value.
-minimumOn :: Ord b => (a -> b) -> [a] -> a
-minimumOn f = minimumBy (compare `on` f)
+--   Raises an error if the list is empty. Only calls the function once per element.
+--
+-- > minimumOn id [] == undefined
+-- > minimumOn length ["test","extra","a"] == "a"
+minimumOn :: (Partial, Ord b) => (a -> b) -> [a] -> a
+minimumOn f [] = error "Data.List.Extra.minimumOn: empty list"
+minimumOn f (x:xs) = g x (f x) xs
+    where
+        g v mv [] = v
+        g v mv (x:xs) | mx < mv = g x mx xs
+                      | otherwise = g v mv xs
+            where mx = f x
 
 -- | A combination of 'group' and 'sort'.
 --
@@ -489,6 +534,16 @@ drop1 [] = []
 drop1 (x:xs) = xs
 
 
+-- | Equivalent to @dropEnd 1@, but likely to be faster and a single lexeme.
+--
+-- > dropEnd1 ""         == ""
+-- > dropEnd1 "test"     == "tes"
+-- > \xs -> dropEnd 1 xs == dropEnd1 xs
+dropEnd1 :: [a] -> [a]
+dropEnd1 [] = []
+dropEnd1 (x:xs) = foldr (\z f y -> y : f z) (const []) xs x
+
+
 -- | Version on `concatMap` generalised to a `Monoid` rather than just a list.
 --
 -- > mconcatMap Sum [1,2,3] == Sum 6
